#!/usr/bin/perl
# pkt-loss-warning.pl  Andrew Daviel, TRIUMF, 1996,97
# A Perl script for monitoring Internet packet loss
# See pkt-loss-warning.txt

# ** Change $recipient to reflect your organisation !! **

# This script creates datafiles in the "./ping" subdirectory
# to dump the WARN database, use print-warn-db.pl

$libloc = "/usr/local/lib/libwww-perl-0.40" ; unshift(@INC, $libloc);
#require "timelocal.pl" ;
require "wwwdates.pl" ;

# figure out system time for midnight
$date = `/bin/date "+%d %b %Y %H:%M"`;
$gdate = `/bin/date -u "+%d %b %Y %H:%M"`;
$midnight = `/bin/date "+%d %b %Y"`;
$midnight = &wwwdates'get_gmtime($midnight) +
  &wwwdates'get_gmtime($gdate) - &wwwdates'get_gmtime($date) ;
$lmidnight = &wwwdates'wtime($midnight,'') ;
$hour = `/bin/date +%H` ; $min = `/bin/date +%M` ;
# expires at next multiple of 10 minutes past the hour
$min = 10* int($min/10) ;
$expires = $midnight + $hour*3600 + $min*60 + 600 ;
$expires = &wwwdates'wtime($expires,'GMT') ;

$date = `/bin/date` ;

$hour_warn_interval = 1.0 ; # min interval to mail re. loss over 2 hours (days)
$min_warn_interval = 0.5 ; # min interval to mail re. loss over 20 minutes (days)
$all_threshold = 30 ; # threshold for all nodes (%)
$all_warn_interval = 0.3 ; # min interval to mail re. total loss (days)



$count = 10 ; # number of pings per node
# packet size is 2nd argument else 56
$psize = $ARGV[1] ? $ARGV[1] : 56 ;

# comment out to disable FNAL/SLAC style reporting
$ping_data_dir ="/usr/local/etc/htdocs/net/hep" ;

$mailprog = '/usr/lib/sendmail';
$recipient = 'andrew@andrew.triumf.ca,rking@triumf.ca,renee@sitka.triumf.ca';

$debug=$ARGV[2]  ;
$ping = "/usr/etc/ping" ;
if ($debug) { $| = 1 ; }
open (SUMM,">summary.new") || print STDERR "Couldn't open summary file\n"; 
open (SUMM2,">summary2.new") || print STDERR "Couldn't open summary2 file\n"; 
print SUMM<<EOT;
<html><head><title>Packet Loss summary from TRIUMF</title>
<META HTTP-EQUIV="Expires" CONTENT="$expires">
</head><body bgcolor="#ffffff">
<h2>Packet Loss summary from TRIUMF</h2>
Measured from <a href="/">sitka.triumf.ca</a><br>
at $date
<i>Updated every 10 minutes</i><br>
(please wait for table)<br>
<table border=1 cellspacing=1 cellpadding=1>
<tr><th>Address</th><th colspan=3>% Packet Loss</th><th colspan=2>Delay (ms)</th>
<th>Hops (255-ttl)</th></tr>
<tr><th>(about node)</th><th>recent</th><th>2 hours</th>
<th>min.</th><th>avg.</th><th>max.</th><th>recent</th></tr>
EOT
print SUMM2<<EOT;
<html><head><title>Packet Loss summary from TRIUMF</title>
<META HTTP-EQUIV="Expires" CONTENT="$expires">
</head><body bgcolor="#ffffff">
<h2>Packet Loss summary from TRIUMF</h2>
Measured from <a href="/">sitka.triumf.ca</a><br>
at $date
<i>Updated every 10 minutes</i><br>
(please wait for table)<br>
<table border=1 cellspacing=1 cellpadding=1>
<tr><th>Address</th><th colspan=3>% Packet Loss</th><th colspan=4>Delay (ms)</th>
<th colspan=2>Hops (255-ttl)</th></tr>
<tr><th>(about node)</th><th>recent</th><th>2 hours</th><th>Last 24 hrs</th>
<th>min.</th><th>avg.</th><th>Last 24 hrs</th><th>max.</th>
<th>recent</th><th>Last 24 hrs</th></tr>
EOT
unless (dbmopen(%WARN,"warned",0644)) { print STDERR "Couldn't open warned database\n"; }
unless (dbmopen(%NODE,"nodenames",0644)) { print STDERR "Couldn't open nodenames database\n"; }
# read node list from $ARGV[0]
open (IN,$ARGV[0]) || die "Can't open $ARGV[0]" ;
while (<IN>) {
  chop ; tr / //s ; s/^ //;
  if (/^#/) { next ; } # comment only
  s/#.*// ; # strip comments
  ($address,$threshold,$threshold_m) = split(/ /) ;
  if (!$address) { next ; }

  ($name,$aliases,$addrtype,$length,@addrs) =gethostbyname($address) ;
  if (!$length) {
    $host = $NODE{$address} ;
    if (!$host) { print STDERR "Couldn't resolve address for $address\n"; }
  } else { 
    @f = unpack('C4',$addrs[0]);
    $host = join('.',@f) ;
    $NODE{$address} = $host ;
  }

  $date = `date +%j_%H_%M_%S` ; @date = split('_',$date) ;
  $date = $date[0] + $date[1]/24 + $date[2]/(24*60) + $date[3]/(24*60*60) ;
  $now = sprintf("%009.5f",$date) ;
  $hits=0 ; $lost=0 ;
# example line
# 156.14748 4 0 # 192.68.61.1
  $file = "ping/".$address ;
  undef(@line) ; $n_hour = 0 ; $hour_loss = 0 ; $n_m10 = 0 ; $m10_loss = 0 ;
  undef(@floss) ; undef(@avrg) ; undef(@hops) ;
  if (open (PING,$file)) {
    while (<PING>) {
      chop ; tr / //s ; s/^ //;
      ($ftime,$floss,$avrg,$hops) = split(/ /) ;
      if ($ftime>=366) { $ftime = $ftime - 366 ; }
      $_ = join(' ',$ftime,$floss,$avrg,$hops) ;
      if (($ftime > ($now-1)) && ($ftime < 367) ) {
        push(@line,$_) ; push(@floss,$floss) ; push(@avrg,$avrg) ;
        push(@hops,$hops) ;
      }
      if ($ftime > ($now-2/24)) {
        $n_hour++ ; $hour_loss = $hour_loss + $floss ;
      }
      if ($ftime > ($now- (1/24)*(2/6))) {
        $n_m10++ ; $m10_loss = $m10_loss + $floss ;
      }
    }
    close(<PING>) ;
  } 
  $nloss = @floss+0 ; $navrg = @avrg+0 ; $nhops = @hops+0 ;
  $min = 0 ; $max = 0 ; $avg = 0 ; $time = time ; 
  for ($i=1;$i<=$count;$i++) {
    $_ = `$ping -c 1 -s $psize  $host` ; # was -q
    if ($debug>1) { print $_ ; }

    $pingd = $_ ;
    chop ; @L = split(/\n/) ;
    foreach $_ (@L) {
      if (!/ttl=/) { next ; }
      s/.*\sttl=// ; s/\s.*// ;
      $ttl = $_ ; last ; 
    }
    if ($ttl && $ttl>220) { $hops = 255-$ttl ;
    } else { $hops = '*' ; }
    $_ = $pingd ;

    tr/\n/ /; s/.*round-trip// ;
    s/.*=// ; s/ms// ; tr/ //d ;
    if (!/d,0packet/) {
      $hits++ ; $all_hits++ ;
      s%/.*%% ;
      $min = (!$min || $_<$min) ? $_ : $min ;
      $max = (!$max || $_>$max) ? $_ : $max ; 
      $avg+= $_ ; $k++ ;
    } else {
      $lost++ ; $all_lost++ ;
    }
  }
  if ($hits) { $avg = sprintf("%.2f",$avg/$hits) ; }
  if ($debug>1) { print "min/avg/max $min/$avg/$max\n"; }
  $loss = int( 100 * $lost / ($hits + $lost) ) ;
  $file = ">ping/".$address ;
  if (open (PING,$file)) {
    foreach $line (@line) {
      print PING "$line\n";
    }
    print PING "$now $loss $avg $hops\n" ;
    push(@floss,$loss) ; push(@avrg,$avg) ; push(@hops,$hops) ;
    close (PING) ;
  }else { print STDERR "Can't write $file\n"; }
  $n_hour++ ; $hour_loss = $hour_loss + $loss ;
  $n_m10++ ; $m10_loss = $m10_loss + $loss ;
  $hour_loss = $hour_loss/$n_hour ;
  $hour_loss = int($hour_loss) ;
  $m10_loss = $m10_loss/$n_m10 ;
  if($debug) { print "Average loss to $address last 2 hours $hour_loss %\n"; }
  if($debug) { print "Average loss to $address last 20 minutes $m10_loss %\n"; }

#  if ($address =~ /^www/i) { 
#    print SUMM "<tr><td><a href=\"http://$address\">$address</a></td>" ;
#  } else {
#    print SUMM "<tr><td>$address</td>" ;
#  }
  print SUMM "<tr><td><a href=\"/cgi-bin/node-info?$address\">$address</a></td>" ;
  print SUMM2 "<tr><td><a href=\"/cgi-bin/node-info?$address\">$address</a></td>" ;
  $xloss = sprintf("%2.2x",int($loss*255/100)) ;
  $xloss1 = sprintf("%2.2x",int(255-$loss*255/100)) ;
  $xhloss = sprintf("%2.2x",int($hour_loss*255/100)) ;
  $xhloss1 = sprintf("%2.2x",int(255-$hour_loss*255/100)) ;
  print SUMM "<td bgcolor=#$xloss$xloss1"."ff>" ;
  print SUMM2 "<td bgcolor=#$xloss$xloss1"."ff>" ;
#  if ($loss == 0) {  print SUMM "<td bgcolor=#90ee90>" ; }
#  elsif ($loss == 100) {  print SUMM "<td bgcolor=#ff6347>" ; }
#  else {  print SUMM "<td bgcolor=#f4a460>" ; }
  print SUMM "\n<a href=/cgi-bin/wrap-plot-day-pkt2/$address>$loss</td>" ;
  print SUMM2 "\n<a href=/cgi-bin/wrap-plot-day-pkt2/$address>$loss</td>" ;
#  if ($hour_loss == 0) {  print SUMM "<td bgcolor=#90ee90>" ; }
#  elsif ($hour_loss == 100) {  print SUMM "<td bgcolor=#ff6347>" ; }
#  else {  print SUMM "<td bgcolor=#f4a460>" ; }
  print SUMM "<td bgcolor=#$xhloss$xhloss1"."ff>" ;
  print SUMM2 "<td bgcolor=#$xhloss$xhloss1"."ff>" ;
  print SUMM "\n<a href=/cgi-bin/wrap-plot-day-pkt2-smooth/$address>$hour_loss</td>\n" ;
  print SUMM2 "\n<a href=/cgi-bin/wrap-plot-day-pkt2-smooth/$address>$hour_loss</td>\n" ;
# <td width=$nloss height=??>
  print SUMM2 "<td>\n" ;
  $i=0 ; $y = 0 ;
  foreach $floss (@floss) {
    $y += $floss ; $i++ ;
    if ($i>=2) {
      $y = int($y/20) ;
      print SUMM2 "<img src=r.gif width=1 height=$y\n>" ;
      $i = 0 ; $y = 0 ;
    }
  }
  print SUMM2 "</td>\n" ;
  print SUMM "<td>$min</td><td>\n" ;
  print SUMM2 "<td>$min</td><td>\n" ;
  print SUMM "<a href=/cgi-bin/wrap-plot-day-avg2/$address>$avg</td>\n";
  print SUMM2 "<a href=/cgi-bin/wrap-plot-day-avg2/$address>$avg</td>\n";
  print SUMM2 "<td>\n" ;
  $i=0 ; $y = 0 ;
  foreach $avrg (@avrg) {
    $y += $avrg ; $i++ ;
    if ($i>=2) {
      $y = int($y/50) ;
      print SUMM2 "<img src=r.gif width=1 height=$y\n>" ;
      $i = 0 ; $y = 0 ;
    }
  }
  print SUMM2 "</td>\n" ;
  print SUMM "<td>$max</td>\n" ;
  print SUMM2 "<td>$max</td>\n" ;
  print SUMM "<td><a href=/cgi-bin/wrap-plot-day-hops2/$address>$hops</a></td>\n" ;
  print SUMM2 "<td><a href=/cgi-bin/wrap-plot-day-hops2/$address>$hops</a></td>\n<td>\n" ;
  $i=0 ; $y = 0 ;
  foreach $fhops (@hops) {
    $y += $fhops ; $i++ ;
    if ($i>=2) {
      $y = int($y/2) ;
      print SUMM2 "<img src=r.gif width=1 height=$y\n>" ;
      $i = 0 ; $y = 0 ;
    }
  }
  print SUMM2 "</td>\n" ;
  print SUMM "</tr>\n\n"; 
  print SUMM2 "</tr>\n\n"; 
  $m10_loss = int($m10_loss) ;
  if ($WARN{$address} > $now) { delete($WARN{$address}) ; }
  if ($hour_loss>$threshold && 
      $WARN{$address} < ($now-$hour_warn_interval)) {
    push (@warn,"Packet loss to ".$address." ".$hour_loss." % in last 2 hours, over threshold ".$threshold."\n");
    $WARN{$address} = $now ;
    $nwarn++ ; $warnadr = $address ;
  }
  $address20 = $address."-20" ;
  if ($WARN{$address20} > $now) { delete($WARN{$address20}) ; }
  if ($m10_loss>$threshold_m && 
      $WARN{$address20} < ($now-$min_warn_interval)) {
    push (@warn,"Packet loss to ".$address." ".$m10_loss." % in last 20 minutes, over threshold ".$threshold_m."\n");
    $WARN{$address20} = $now ;
    $nwarn++ ; $warnadr = $address20 ;
  }

}
print SUMM<<EOT;
</table><br>
See also <a href="summary2.html">version with inline graphs</a> (~150kb).
Also <a href="summary2.html.gz">gzipped HTML</a> (~4kb).
<p>
<a href="nodes.html">Node Map</a> (July 97)
<p>
<a href="http://sitka.triumf.ca/net/pkt-loss-warning.tar.gz">Download the script</a>
</body></html>
EOT
print SUMM2<<EOT;
</table><br>
Links for each node give DNS lookup, etc. Links on packet loss numbers, delay
and hops give a graph for the past 24 hours. The graphs are generated
dynamically, so may take several seconds to appear.
<p>
<a href="summary.html">no-graph version</a>
<p>
<a href="nodes.html">Node Map</a> (July 97)
<p>
<a href="pkt-loss-warning.tar.gz">Download the script</a>
</body></html>
EOT
close(SUMM) ; system("mv -f summary.new summary.html") ;
close(SUMM2) ; system("cp  summary2.new summary2.html ; gzip summary2.new ; mv summary2.new.gz summary2.html.gz") ;
open (META,">summary.html.meta") ;
print META "Expires: $expires\n";
close(META) ;
open (META,">summary2.html.gz.meta") ;
print META "Expires: $expires\n";
close(META) ;

$all_loss = int( 100 * $all_lost / ($all_hits + $all_lost) ) ;
if ($WARN{"ALL"} > $now) { delete($WARN{"ALL"}) ; }
if ($all_loss>$all_threshold && 
    $WARN{"ALL"} < ($now-$all_warn_interval)) {
  push (@warn,"Packet loss to all nodes ".$all_loss." % at last pass, , over threshold ".$all_threshold."\n");
  $WARN{"ALL"} = $now ; $allwarn++ ;
}
if ($allwarn) {
  $subject = "Excessive Packet Loss" ;
} elsif ($nwarn == 1) {
  $subject = "Packet Loss to ".$warnadr ;
} else {
  $subject = "Packet Loss to 2 or more nodes" ;
}
if (@warn+0) {
  open (MAIL, "|$mailprog $recipient") || die "Can't open $mailprog!\n";
  print MAIL "Subject: $subject\n\n";
  foreach $line (@warn) {
    print MAIL "$line";
  }
  print MAIL "\nFor recent data see http://sitka.triumf.ca/net/summary.html

Warnings relating to packet loss over the 2-hour threshold are sent at most 
every $hour_warn_interval day(s) for each new node.
Warnings for loss over the 20-minute threshold are sent at most every 
$min_warn_interval days, while warnings for high global loss are sent
every $all_warn_interval days.\n";
  close (MAIL);
}

